from zope.component import getUtility
from Products.GenericSetup.tests.common import DummyImportContext
from Products.GenericSetup.tests.common import DummyExportContext

from Products.TinyMCE.interfaces.utility import ITinyMCE
from Products.TinyMCE.exportimport import importTinyMCESettings
from Products.TinyMCE.exportimport import exportTinyMCESettings
from Products.TinyMCE.tests.base import FunctionalTestCase


class ImportExportTestCase(FunctionalTestCase):

    def test_import_export(self):
        # This module takes care of exporting and importing settings of TinyMCE. Let's
        # create a dummy import context. And a dummy file.

        xml = """\
        <object>
          <resourcetypes>
            <link_using_uids value="True"/>
            <customplugins purge="True">
              <element value="testplugin"/>
            </customplugins>
            <linkable purge="False">
              <element value="Topic"/>
            </linkable>
            <customplugins purge="True">
              <element value="testplugin"/>
            </customplugins>
          </resourcetypes>
        </object>
        """

        context = DummyImportContext(self.portal, purge=True)
        context._files = {'tinymce.xml': xml}

        # Now import the file.
        importTinyMCESettings(context)

        # Our specified plugin should now be stored in the utility.

        tinymce_utility = getUtility(ITinyMCE)
        self.assertIn('testplugin', tinymce_utility.customplugins)

        # Let's create a dummy export context.
        context = DummyExportContext(self.portal)

        # And export the current settings.
        exportTinyMCESettings(context)

        # Check if tinymce.xml is exported.
        self.assertEqual(context._wrote[0][0], 'tinymce.xml')

        # Check the contents of the export.
        self.assertIn('testplugin', context._wrote[0][1])



# -*- coding: utf-8 -*-

import unittest2 as unittest

from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.testing import z2


class TinyMCELayer(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        """Set up Zope."""
        import Products.TinyMCE
        self.loadZCML(package=Products.TinyMCE)
        z2.installProduct(app, 'Products.TinyMCE')

    def setUpPloneSite(self, portal):
        """Set up Plone."""
        # Install into Plone site using portal_setup
        self.applyProfile(portal, 'Products.TinyMCE:TinyMCE')

    def tearDownZope(self, app):
        """Tear down Zope."""
        z2.uninstallProduct(app, 'Products.TinyMCE')


FIXTURE = TinyMCELayer()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,), name="TinyMCELayer:Integration")
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,), name="TinyMCELayer:Functional")


class BaseTestCase(unittest.TestCase):

    def setUp(self):
        self.portal = self.layer['portal']
        self.app = self.layer['app']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)


class IntegrationTestCase(BaseTestCase):
    """Base class for integration tests."""

    layer = INTEGRATION_TESTING


class FunctionalTestCase(BaseTestCase):
    """Base class for functional tests."""

    layer = FUNCTIONAL_TESTING



# Check for Plone versions
try:
    from plone.app.upgrade import v40
    HAS_PLONE30 = True
    HAS_PLONE40 = True
except ImportError:
    HAS_PLONE40 = False
    try:
        from Products.CMFPlone.migrations import v3_0
    except ImportError:
        HAS_PLONE30 = False
    else:
        HAS_PLONE30 = True


# BBB for Z2 vs Z3 interfaces checks
def implementedOrProvidedBy(anInterface, anObject):
    if HAS_PLONE40:
        return anInterface.providedBy(anObject)
    else:
        return anInterface.isImplementedBy(anObject)



import transaction
from plone.testing.z2 import Browser

from Products.TinyMCE.tests.base import IntegrationTestCase
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD


class WysiwygSupportTestCase(IntegrationTestCase):

    def setUp(self):
        super(WysiwygSupportTestCase, self).setUp()
        # Unhide exceptions.

    def test_personalized(self):
        self.portal.error_log._ignored_exceptions = ()
        browser = Browser(self.app)
        portal_url = self.portal.absolute_url()

        # Get an account and login via the login form.
        browser.open(portal_url + '/login_form')
        browser.getControl(name='__ac_name').value = TEST_USER_NAME
        browser.getControl(name='__ac_password').value = TEST_USER_PASSWORD
        browser.getControl(name='submit').click()

        # Create a blank page for test edits. (front-page is too noisy.)
        testpage = self.portal.absolute_url() + '/' + self.portal.invokeFactory('Document', 'testpage') + '/edit'
        self.assertEqual(testpage, 'http://nohost/plone/testpage/edit')
        transaction.commit()

        # Set up personalize_form
        personalizer = portal_url + '/personalize_form'
        self.assertEqual(personalizer, 'http://nohost/plone/personalize_form')
        browser.open(personalizer)
        browser.getControl(name='email').value = 'test@example.org'
        browser.getControl(name='form.button.Save').click()
        self.assertIn('<dd>Your personal settings have been saved.</dd>', browser.contents)

        #Test different editors
        #----------------------
        #Which editor is used is set globally and can be overridden in the personal preferences.
        #This results in the following matrix:
        #=====       =====                   ======
        #Global      Personal                Expected
        #=====       =====                   ======
        #TinyMCE     Use site's default      TinyMCE
        #TinyMCE     None                    Basic textarea
        #TinyMCE     TinyMCE                 TinyMCE
        #<empty>     Use site's default      Basic textarea
        #<empty>     None                    Basic textarea
        #<empty>     TinyMCE                 TinyMCE
        #=====       =====                   ======

        # Set the editor globally to TinyMCE:
        self.portal.portal_properties.site_properties.default_editor = 'TinyMCE'
        transaction.commit()

        # If the user sets 'Use site's default'...
        browser.open(personalizer)
        browser.getControl(name='wysiwyg_editor').value = ['']
        browser.getControl(name='form.button.Save').click()
        self.assertIn('<dd>Your personal settings have been saved.</dd>', browser.contents)

        # we should get TinyMCE:
        browser.open(testpage)
        self.assertIn('TinyMCEConfig', browser.contents)

        # If the user sets 'None'...
        browser.open(personalizer)
        browser.getControl(name='wysiwyg_editor').value = ['None']
        browser.getControl(name='form.button.Save').click()
        self.assertIn('<dd>Your personal settings have been saved.</dd>', browser.contents)

        # We should get just a textarea:
        browser.open(testpage)
        self.assertIn('<textarea name="text" rows="25" id="text"', browser.contents)

        # If the user sets 'TinyMCE'...
        browser.open(personalizer)
        browser.getControl(name='wysiwyg_editor').value = ['TinyMCE']
        browser.getControl(name='form.button.Save').click()
        self.assertIn('<dd>Your personal settings have been saved.</dd>', browser.contents)

        # we should get TinyMCE:
        browser.open(testpage)
        self.assertIn('TinyMCEConfig', browser.contents)

        # Set the editor globally to nothing:
        self.portal.portal_properties.site_properties.default_editor = ''
        transaction.commit()

        # If the user sets 'Use site's default'...
        browser.open(personalizer)
        browser.getControl(name='wysiwyg_editor').value = ['']
        browser.getControl(name='form.button.Save').click()
        self.assertIn('<dd>Your personal settings have been saved.</dd>', browser.contents)

        # we should get just a textarea:
        browser.open(testpage)
        self.assertIn('<textarea name="text" rows="25" id="text"', browser.contents)

        # If the user sets 'None'...
        browser.open(personalizer)
        browser.getControl(name='wysiwyg_editor').value = ['None']
        browser.getControl(name='form.button.Save').click()
        self.assertIn('<dd>Your personal settings have been saved.</dd>', browser.contents)

        # we should get just a textarea:
        browser.open(testpage)
        self.assertIn('<textarea name="text" rows="25" id="text"', browser.contents)

        # If the user sets 'TinyMCE'...
        browser.open(personalizer)
        browser.getControl(name='wysiwyg_editor').value = ['TinyMCE']
        browser.getControl(name='form.button.Save').click()
        self.assertIn('<dd>Your personal settings have been saved.</dd>', browser.contents)

        # we should get TinyMCE:
        browser.open(testpage)
        self.assertIn('TinyMCEConfig', browser.contents)



from Acquisition import aq_inner
from Acquisition import aq_parent
from zExceptions import BadRequest
from zope.interface import implements
from zope.component import getUtility

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces._content import IFolderish
from plone.outputfilters.browser.resolveuid import uuidFor

from Products.TinyMCE.interfaces.utility import ITinyMCE
from Products.TinyMCE.adapters.interfaces.Upload import IUpload
from Products.TinyMCE import TMCEMessageFactory as _
from zope.i18n import translate

TEMPLATE = """
<html>
<head></head>
<body onload="%s"></body>
</html>
"""


class Upload(object):
    """Adds the uploaded file to the folder"""
    implements(IUpload)

    def __init__(self, context):
        """Constructor"""

        self.context = context

    def errorMessage(self, msg):
        """Returns an error message"""
        msg = translate(msg, context=self.context.REQUEST)
        script = TEMPLATE % ("window.parent.uploadError('" + msg.replace("'", "\\'") + "');")
        return script

    def okMessage(self, msg):
        """Returns an ok message"""
        script = TEMPLATE % ("window.parent.uploadOk('" + msg.replace("'", "\\'") + "');")
        return script

    def cleanupFilename(self, name):
        """Generate a unique id which doesn't match	the system generated ids"""

        context = self.context
        id = ''
        name = name.replace('\\', '/')  # Fixup Windows filenames
        name = name.split('/')[-1]  # Throw away any path part.
        for c in name:
            if c.isalnum() or c in '._':
                id += c

        # Raise condition here, but not a lot we can do about that
        if context.check_id(id) is None and getattr(context, id, None) is None:
            return id

        # Now make the id unique
        count = 1
        while 1:
            if count == 1:
                sc = ''
            else:
                sc = str(count)
            newid = "copy%s_of_%s" % (sc, id)
            if context.check_id(newid) is None and getattr(context, newid, None) is None:
                return newid
            count += 1

    def upload(self):
        """Adds uploaded file"""

        object = aq_inner(self.context)
        if not IFolderish.providedBy(object):
            object = aq_parent(object)

        context = self.context
        request = context.REQUEST
        ctr_tool = getToolByName(self.context, 'content_type_registry')
        utility = getUtility(ITinyMCE)

        id = request['uploadfile'].filename
        content_type = request['uploadfile'].headers["Content-Type"]
        typename = ctr_tool.findTypeName(id, content_type, "")

        # Permission checks based on code by Danny Bloemendaal

        # 1) check if the current user has permissions to add stuff
        if not context.portal_membership.checkPermission('Add portal content', context):
            return self.errorMessage(_("You do not have permission to upload files in this folder"))

        # 2) check image types uploadable in folder.
        #    priority is to content_type_registry image type
        allowed_types = [t.id for t in context.getAllowedTypes()]
        if typename in allowed_types:
            uploadable_types = [typename]
        else:
            uploadable_types = []

        if content_type.split('/')[0] == 'image':
            image_portal_types = utility.imageobjects.split('\n')
            uploadable_types += [t for t in image_portal_types
                                    if t in allowed_types
                                       and t not in uploadable_types]

        # Get an unused filename without path
        id = self.cleanupFilename(id)

        title = request['uploadtitle']
        description = request['uploaddescription']

        for metatype in uploadable_types:
            try:
                newid = context.invokeFactory(type_name=metatype, id=id)
                if newid is None or newid == '':
                    newid = id

                obj = getattr(context, newid, None)

                # set primary field
                pf = obj.getPrimaryField()
                pf.set(obj, request['uploadfile'])
                break

            except ValueError:
                continue
            except BadRequest:
                return self.errorMessage(_("Bad filename, please rename."))
        else:
            return self.errorMessage(_("Not allowed to upload a file of this type to this folder"))


        # Set title + description.
        # Attempt to use Archetypes mutator if there is one, in case it uses a custom storage
        if title:
            try:
                obj.setTitle(title)
            except AttributeError:
                obj.title = title

        if description:
            try:
                obj.setDescription(description)
            except AttributeError:
                obj.description = description

        if not obj:
            return self.errorMessage(_("Could not upload the file"))

        obj.reindexObject()

        if utility.link_using_uids:
            return self.okMessage("resolveuid/%s" % (uuidFor(obj)))

        return self.okMessage("%s" % (obj.absolute_url()))

    def setDescription(self, description):
        self.context.setDescription(description)
